/******************************************************************************* 
 * Copyright (c) 2007 Red Hat, Inc. 
 * Distributed under license by Red Hat, Inc. All rights reserved. 
 * This program is made available under the terms of the 
 * Eclipse Public License v1.0 which accompanies this distribution, 
 * and is available at http://www.eclipse.org/legal/epl-v10.html 
 * 
 * Contributors: 
 * Red Hat, Inc. - initial API and implementation 
 ******************************************************************************/ 

package com.jboss.jbds.installer.bean;

import java.io.File;

public class ServerType {
	
	private static final String SYSTEM_JAR_NAME = "twiddle.jar";
	private String type;
	private String jbossSystemJarPath;
	private String[] versions = new String[0];
	
	private ServerType(String type, String jbossSystemJarPath, String[] versions) {
		this.type = type;
		this.jbossSystemJarPath = jbossSystemJarPath;
		this.versions = versions;
	}

	public static final ServerType AS = new ServerType(
			"AS",
			"bin"+File.separatorChar + SYSTEM_JAR_NAME,
			new String[]{"5.0", "4.2", "4.0", "3.2"});
	
	public static final ServerType EAP = new ServerType(
			"EAP",
			"jboss-as" + File.separatorChar + "bin"+ File.separatorChar + SYSTEM_JAR_NAME, 
			new String[]{"4.2","4.3"});
	
	public static final ServerType SOAP = new ServerType(
			"SOA-P",
			"jboss-as" + File.separatorChar + "bin"+ File.separatorChar + SYSTEM_JAR_NAME,
			new String[]{"4.3"});
	
	public static final ServerType UNKNOWN = new ServerType(
			"UNKNOWN",
			"",
			new String[]{"5.0", "4.3", "4.2", "4.0", "3.2"});

	public String toString() {
		return type;
	}
	
	public static ServerType getType(String name) {
		if(AS.type.equals(name)) {
			return AS;
		} else if(EAP.type.equals(name)) {
			return EAP;
		} else if(SOAP.type.equals(name)) {
			return SOAP;
		}
		throw new IllegalArgumentException("Name '" + name + "' cannot be converted to ServerType");
	}

	public String[] getVersions() {
		return versions;
	}
	
	public String getType() {
		return type;
	}
	
	public String getSystemJarPath() {
		return jbossSystemJarPath;
	}
	
	public static final ServerType[] KNOWN_TYPES = {AS, EAP, SOAP};
	
}